<?php
/* eConnect Communication Class

   Used to communicate with an eConnect Server
   
   Instructions: Set properties, such as ip_address, port, ssl, packet_password
                 Set packet_type
                 Set request_array with CODE=>VALUE (no need to set WID, PPW, BUA, D or T)
                 Optionally set override_request_packet instead using an exact, complete pipe-delimited packet string
                 Call send_packet method
                 Will set error_status, error_message, response_array, response_packet (raw)
*/

class econnect_communication  {
    	
   public $ip_address='localhost' ;
   public $port=443 ;
   public $packet_password='' ;
   public $ssl=false ;
   public $request_array=array();
   public $override_request_packet='';
   public $response_packet='';
   public $response_array=array();
   public $error_status=false;
   public $error_message='';
   public $connect_timeout=5;
   public $packet_timeout=10;
   public $packet_type = '9999';
   
    	
   
   /* sendpacket()
   --------------------------------
   This function is the main function used to send packets to the eConnect Server,
   and receive the response.  It checks for errors in sending, and checks any error message
   returned in the packet.
   */
   public function send_packet() {

      $econnect_error_status = false;
      $econnect_error_message = '';
      $econnect_response_packet = '';

      $user_agent = $_SERVER['HTTP_USER_AGENT'];
      $ip_address = $this->get_ip();

      $host = $this->ip_address;
      $port = $this->port;
      $ssl = $this->ssl;
      $connect_timeout = $this->connect_timeout;
      $packet_timeout = $this->packet_timeout;
      
      //make sure timeouts not less than minimum
      if ($connect_timeout < 5) $connect_timeout = 5;
      if ($packet_timeout < 10) $packet_timeout = 10;
      
      //set request packet
      if ($this->override_request_packet != ''){
      	  //Use Override Packet
      	  $econnect_request_packet = $this->override_request_packet;
      }
      else {
         //Create Pipe-delimited eConnect packet from request_packet_array |CODE=VALUE|CODE=VALUE  etc.
         $request_packet_data = '';
         foreach ($this->request_array as $array_key => $array_value ) {
          	 $request_packet_data .= '|'. $array_key . '=' . $array_value ;
         }
    
         $t=time();
         
         //Start with Packet Type
         $econnect_request_packet = $this->packet_type;
         
         //Add Web Id / Packet Password
         if ($this->request_array['WID'] == ''){
            $econnect_request_packet .= '|WID=' . $t .
                                        '|PPW=' . $this->packet_password ;
         }
         else {
            $econnect_request_packet .= '|PPW=' . $this->packet_password ;
         }

         //Add Main Packet Data
         $econnect_request_packet .= $request_packet_data;
         
          
         //Add Browser User Agent/IP/Date/Time
         $econnect_request_packet .= '|BUA=' . $user_agent .
                                     '|RIP=' . $ip_address .
                                     '|D='.date('m/d/y',$t).
                                     '|T='.date('h:iA',$t).chr(4);
                                     
      }


      //connect to server 
      if ($ssl == '1') {
         $socket = @fsockopen('ssl://'.$host, $port, $errno, $errstr, $connect_timeout);
      }
      else {
         $socket = @fsockopen($host, $port, $errno, $errstr, $connect_timeout);
      }

      if (is_resource($socket)) {
         
      
          fwrite($socket, $econnect_request_packet);
          stream_set_timeout($socket, $packet_timeout);

      
          // Get Response
          do
          { 
          $out = fgets($socket,2);
          $info = stream_get_meta_data($socket);
          $econnect_response_packet .= $out;
          $i++;
          } 
          while($out != chr(4) && $i<1000000 && $info['timed_out']==false);
          fclose($socket);

          if ($econnect_response_packet != '') {
            if (strpos($econnect_response_packet,'|S=1') === false ) {
               //Success Flag not returned

               //Get Error Message returned
               $errpos = strpos($econnect_response_packet,'|E=');
               $errend = strpos($econnect_response_packet,'|',$errpos+1);
               $errmsg = substr($econnect_response_packet,$errpos+3,$errend-$errpos-3);

               $econnect_error_message = $errmsg;
               $econnect_error_status = true;
            }
            else {
               //Parse Response
               //a) Explode response by pipe delimiter
               $econnect_temp_array = explode("|",$econnect_response_packet);
               //b) For each item, add to packet array with Key, Value
               $econnect_response_array = array();
               foreach ($econnect_temp_array as $econnect_value_pair) {
                  $econnect_value_pair_array = explode("=",$econnect_value_pair,2);
                  $econnect_response_array[$econnect_value_pair_array[0]] = $econnect_value_pair_array[1];
               }
            	
            }
          }
          else {
            //Invalid response
            $econnect_error_message = ($info['timed_out']==true) ? 'Server Timeout.' : 'Invalid response received from server.';
            $econnect_error_status = true;
          }
      
      } else {
         //Socket connection not made
         $econnect_error_message = 'Server connection could not be made.';
         $econnect_error_status = true;
      }
      
      $this->response_packet = $econnect_response_packet;
      $this->response_array = $econnect_response_array;
      $this->error_status = $econnect_error_status;
      $this->error_message = $econnect_error_message;
      
      return;
 
   
   }


   
   function get_ip() {
         foreach (array('HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR') as $key)
         {
             if (array_key_exists($key, $_SERVER) === true)
             {
                 foreach (array_map('trim', explode(',', $_SERVER[$key])) as $ip)
                 {
                     if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE) !== false)
                     {
                         return $ip;
                     }
                 }
             }
         }
     }
   
 
    	
}


?>